
from .request import Request


def verify(receipt_data, password=None, proxy_url=None, **kwargs):
    """Shortcut API for :class:`itunesiap.request.Request`

    :param str receipt_data: An iTunes receipt data as Base64 encoded string.
    :param proxy_url: A proxy url to access the itunes validation url
    :param bool use_production: Override environment value if given
    :param bool use_sandbox: Override environment value if given
    :param bool verify_ssl: Override environment value if given

    :return: :class:`itunesiap.receipt.Receipt` object if succeed.
    :raises: Otherwise raise a request exception.
    """
    return Request(receipt_data, password, proxy_url).verify(**kwargs)




__all__ = ('Environment', 'default', 'production', 'sandbox', 'review', 'current')


class EnvironmentStack(list):
    def push(self, env):
        self.append(env)


class Environment(object):
    """Environement provides option preset for `Request`. `default` is default"""

    ITEMS = ('use_production', 'use_sandbox')
    _stack = EnvironmentStack()

    def __init__(self, **kwargs):
        self.use_production = kwargs.get('use_production', True)
        self.use_sandbox = kwargs.get('use_sandbox', False)
        self.verify_ssl = kwargs.get('verify_ssl', True)

    def clone(self, **kwargs):
        options = self.extract()
        options.update(**kwargs)
        return self.__class__(**options)

    def push(self):
        self._stack.push(self)

    @classmethod
    def pop(self):
        self._stack.pop()

    def __enter__(self):
        self._ctx_id = len(self._stack)
        self.push()
        return self

    def __exit__(self, exc_type, exc_value, tb):
        self._stack.pop(self._ctx_id)

    @classmethod
    def current(cls):
        return cls._stack[-1]

    def override(self, **kwargs):
        """Override options in kwargs to given object `self`."""
        for item in self.ITEMS:
            if item in kwargs:
                setattr(self, item, kwargs[item])

    def extract(self):
        """Extract options from `self` and merge to `kwargs` and return new object."""
        options = {}
        for item in self.ITEMS:
            options[item] = getattr(self, item)
        return options

default = Environment(use_production=True, use_sandbox=False, verify_ssl=True)
production = Environment(use_production=True, use_sandbox=False, verify_ssl=True)
sandbox = Environment(use_production=False, use_sandbox=True, verify_ssl=True)
review = Environment(use_production=True, use_sandbox=True, verify_ssl=True)

unsafe = Environment(use_production=True, use_sandbox=True, verify_ssl=False)


default.push()


def current():
    return Environment.current()




import warnings
import functools


class lazy_property(object):
    '''http://stackoverflow.com/questions/3012421/python-lazy-property-decorator
    '''

    def __init__(self, function):
        self.function = function

    def __get__(self, obj, cls):
        value = self.function(obj)
        setattr(obj, self.function.__name__, value)
        return value


def deprecated(func):
    '''https://wiki.python.org/moin/PythonDecoratorLibrary#Generating_Deprecation_Warnings

    This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emitted
    when the function is used.'''

    @functools.wraps(func)
    def new_func(*args, **kwargs):
        warnings.warn_explicit(
            "Call to deprecated function {}.".format(func.__name__),
            category=UserWarning,
            filename=func.func_code.co_filename,
            lineno=func.func_code.co_firstlineno + 1
        )
        return func(*args, **kwargs)
    return new_func



"""
itunes-iap
~~~~~~~~~~

Itunes In-app Purchase verification api.

:copyright: (c) 2013 Jeong YunWon - 2014 Andy Briggs
:license: 2-clause BSD.
"""

from six import PY3
import pkg_resources

from .request import Request
from .receipt import Response, Receipt, InApp
from .shortcut import verify

from . import exceptions
from . import environment

exc = exceptions
env = environment  # env.default, env.sandbox, env.review

__version__ = pkg_resources.resource_string('itunesiap', 'version.txt').strip()
if PY3:
    __version__ = __version__.decode('ascii')
VERSION = tuple(int(v) for v in __version__.split('.'))

__all__ = (
    'Request', 'Response', 'Receipt', 'InApp', 'verify',
    'exceptions', 'exc', 'environment', 'env')



from __future__ import with_statement

from setuptools import setup


def get_version():
    with open('itunesiap/version.txt') as f:
        return f.read().strip()


def get_readme():
    try:
        with open('README.rst') as f:
            return f.read().strip()
    except IOError:
        return ''


setup(
    name='itunes-iap',
    version=get_version(),
    description='Apple iTunes In-app purchase verification api.',
    long_description=get_readme(),
    author='Jeong YunWon',
    author_email='itunesiap@youknowone.org',
    url='https://github.com/youknowone/itunes-iap',
    packages=(
        'itunesiap',
    ),
    package_data={
        'itunesiap': ['version.txt']
    },
    install_requires=[
        'requests[security]', 'prettyexc>=0.6.0', 'six', 'pytz', 'python-dateutil',
    ],
    tests_require=[
        'pytest', 'tox', 'mock', 'patch',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
